#include <stdio.h>
#include <pthread.h>
#include <signal.h>
#include <alsa/asoundlib.h>

#define W 1280
#define H 720

unsigned char frame[H][W][3] = {0};

int starts = 0, startf = 0;
int currents = 0, currentf = 0;
int stops = 0, stopf = 0, nbframes = 0;
int maxframe = 24, sec, fra;
int stopped = 1, quit = 0, encode_started = 0;
int isrecording = 0, framestorec = 0;
float start = 0, length = 0;

snd_seq_t *seq_handle = NULL;

pthread_t midiThread, imageThread;


void *Midi () {
  while (!quit) { 
     snd_seq_event_t *ev = NULL;
     snd_seq_event_input(seq_handle, &ev);

     if (ev->type == SND_SEQ_EVENT_SYSEX) {

         int cmd = ((unsigned char*)ev->data.ext.ptr)[2];
         int psc = ((unsigned char*)ev->data.ext.ptr)[4];
         int tc = ((unsigned char*)ev->data.ext.ptr)[3];
         int ln = ev->data.ext.len;

	 for (int i = 0; i < ev->data.ext.len; ++i) printf(" %02X", ((unsigned char*)ev->data.ext.ptr)[i]); printf("\n");

          if (cmd == 0x7F) {
            
            if (ln == 6) // Play & Stop
             {
               if (psc == 6) {
                 // maxframe = 0;
                 startf = -1;
                 if (!stopped) psc = 1; 
                 encode_started = 0;
                 isrecording = 1;
                 stopped = 0;
                 framestorec = 0;
                 printf ("Play\n");
                }
               if (psc == 1) {
                 stops = sec; stopf = fra;
                 printf ("%i, %i\n", stops, stopf);
                 start = starts + (startf / maxframe);
                 length = stops + (stopf / maxframe) - start;
                 isrecording = 0; framestorec = 0;

                 stopped = 1;
                 startf = -1;
                 printf ("Stop\n");
                }
             }
            else if ((ln == 9) && (tc == 0x01)) {
              sec = (((unsigned char*)ev->data.ext.ptr)[5] * 60) + ((unsigned char*)ev->data.ext.ptr)[6];
              fra = ((unsigned char*)ev->data.ext.ptr)[7];
              // if (!isrecording) if (!fra && (maxframe >= 24)) isrecording = 1;
              if (isrecording) framestorec++; 
    
              if (startf == -1) { starts = sec; startf = fra; }
              if (fra > maxframe) maxframe = fra; 

              printf ("TC %i:%i:%i/%i\n", sec/60, sec%60, fra, maxframe);
             } 
           } 
	}
     }
  pthread_exit(EXIT_SUCCESS);
}

FILE *pipein;
FILE *pipeout;


int Encode (int s) {
     if (s) { nbframes = 0; pipeout = popen("ffmpeg -y -f rawvideo -vcodec rawvideo -pix_fmt rgb24 -s 1280x720 -r 24 -i - -f mp4 -q:v 5 -an -vcodec mpeg4 temp.mp4", "w");  encode_started = 1; }
     else { fflush(pipeout); pclose(pipeout); encode_started = 0;
            char buffer[1000];
            sprintf(buffer, "mv temp.mp4 output_start-%isec%ifr.mp4", starts, startf); 
            system (buffer);
           }
}


void *Image () {
 
   int x, y, count;

   while (!quit) {

    while (stopped) {}
    
    while(!stopped) {
     count = fread(frame, 1, H*W*3, pipein); if (count != H*W*3) break;
    /*    for (y=0 ; y<H ; ++y) for (x=0 ; x<W ; ++x) {  frame[y][x][0] = 255 - frame[y][x][0]; frame[y][x][1] = 255 - frame[y][x][1]; frame[y] [x][2] = 255 - frame[y][x][2]; }  */  
     if (framestorec) { fwrite(frame, 1, H*W*3, pipeout); framestorec--; }
    }

    Encode (0); 
    Encode (1);
  
    printf ("Start: %f, Length: %f, Frames:%i\n", start, length, nbframes);

   }
  pthread_exit(EXIT_SUCCESS);
}


int Cam (int s) {
// if (s) pipein = popen("ffmpeg -i base.avi -f image2pipe -vcodec rawvideo -pix_fmt rgb24 -", "r");
 if (s) pipein = popen ("ffmpeg -video_size 1280x720 -framerate 30 -f x11grab -i :0.0+00,00 -f image2pipe -vcodec rawvideo -pix_fmt rgb24 -", "r");
 else { fflush(pipein); pclose(pipein); }
}


int End () {
 Cam (0);
 pthread_cancel(midiThread);
 pthread_cancel(imageThread);
}


static void sigkill(int p) { quit = 1; End (); }


int main(void)
{
    snd_seq_open(&seq_handle, "default", SND_SEQ_OPEN_INPUT, 0);
    snd_seq_set_client_name(seq_handle, "video recorder");
    snd_seq_create_simple_port(seq_handle, "in", SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE, SND_SEQ_PORT_TYPE_APPLICATION);
    
    signal(SIGTSTP,&sigkill); signal(SIGINT,&sigkill);

    Cam (1);
    Encode (1);

    if (pthread_create(&midiThread, NULL, Midi, NULL)) exit (1);
    if (pthread_create(&imageThread, NULL, Image, NULL)) exit (2);

    int e = getchar ();

    End ();

    return 0;
}